@@ -25,13 +25,29 @@ def test_consumer_config():
     )
 
 
+def test_redact_password_from_str_output():
+    bluesky_consumer = BlueskyConsumer(
+        topics=["test.redact.password"],
+        bootstrap_servers="1.2.3.4:9092",
+        group_id="test-redact-password-group",
+        consumer_config={
+            "sasl.password": "PASSWORD",
+        },
+    )
+
+    bluesky_consumer_str_output = str(bluesky_consumer)
+    assert "PASSWORD" not in bluesky_consumer_str_output
+    assert "sasl.password" in bluesky_consumer_str_output
+    assert "****" in bluesky_consumer_str_output
+
+
 def test_bad_consumer_config():
     test_topic = "test.bad.consumer.config"
     with pytest.raises(ValueError) as excinfo:
         BlueskyConsumer(
             topics=[test_topic],
             bootstrap_servers="1.2.3.4:9092",
-            group_id="abc",
+            group_id="test-bad-consumer_config",
             consumer_config={
                 "bootstrap.servers": "5.6.7.8:9092",
                 "auto.offset.reset": "latest",
